from typing import Optional
from urllib.parse import urlparse, parse_qs, quote
from base64 import b64encode
from datetime import datetime, timedelta

from buptmw.constants import UCLOUD as UCLOUDE
from buptmw.plates.cas import CAS
from buptmw.plates.template import Module_CAS


class UCloud(Module_CAS):
    def __init__(self, cas: Optional[CAS] = None):
        super().__init__(cas)
        self._login()

    def _get_cookies(self):
        info = self.get(UCLOUDE.INFO).json()["data"]
        current = self.get(UCLOUDE.CURRENT).json()["data"]
        user = self.get(UCLOUDE.USER).json()["data"]

        cookies = {}
        cookies["iClass-uuid"] = self.user_id
        cookies["iClass-token"] = self.access_token
        cookies["iClass-refresh_token"] = self.refresh_token
        cookies["iClass-login-meth"] = "icloud"
        cookies["iClass-expert-account"] = self.user_name
        cookies["iClass-real_name"] = quote(self.real_name)
        cookies["iClass-role_name"] = self.role_name
        cookies["iClass-avatar"] = self.avatar
        cookies["iClass-loginId"] = self.loginId
        cookies["iClass-user-info"] = quote(str(info))
        cookies["iClass-current-term"] = quote(str(current))
        cookies["iClass-identity"] = self.identity
        cookies["iClass-user-role"] = quote(str(user[0]))
        cookies["iClass-login-roles"] = quote(str(user))

        for cookie in cookies:
            self.cookies.set(
                name=cookie,
                value=cookies[cookie],
                domain="ucloud.bupt.edu.cn",
                expires=(datetime.now() + timedelta(hours=1)).timestamp()
            )

    def _login(self):
        self.headers["Authorization"] = "Basic " + b64encode("portal:portal_secret".encode()).decode()
        resp = self.get(UCLOUDE.LOGIN)

        self.ticket = parse_qs(urlparse(resp.url).query)["ticket"][0]
        resp = self.post(
            UCLOUDE.TOKEN,
            data={
                "ticket": self.ticket,
                "grant_type": "third"
            }
        )
        data = resp.json()
        
        self.user_id = data["user_id"]
        self.access_token = data["access_token"]
        self.refresh_token = data["refresh_token"]
        self.role_name = data["currentRole"]
        self.loginId = data["loginId"]
        self.user_name = data["user_name"]
        self.real_name = data["real_name"]
        self.avatar = data["avatar"]
        self.dept_id = data.get("dept_id", "undefinded")
        self.identity = f"{self.role_name}:{self.dept_id}"

        self.headers["Blade-Auth"] = self.access_token
        self._get_cookies()



from enum import StrEnum


class SESSION(StrEnum):
    USER_AGENT = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36"


class CAS(StrEnum):
    LOGIN = "https://auth.bupt.edu.cn/authserver/login"


class UC(StrEnum):
    LOGIN = "https://uc.bupt.edu.cn/api/login?target=https://uc.bupt.edu.cn/#/user/login"
    CHECK = "https://uc.bupt.edu.cn/api/uc/status"


class UCLOUD(StrEnum):
    LOGIN = "https://auth.bupt.edu.cn/authserver/login?service=https://ucloud.bupt.edu.cn"
    TOKEN = "https://apiucloud.bupt.edu.cn/ykt-basics/oauth/token"
    INFO = "https://apiucloud.bupt.edu.cn/ykt-basics/info"
    CURRENT = "https://apiucloud.bupt.edu.cn/ykt-site/base-term/current"
    USER = "https://apiucloud.bupt.edu.cn/ykt-basics/userroledomaindept/listByUserId"
    CHECK = "https://apiucloud.bupt.edu.cn/blade-portal/home-page-info/getShufflingWebList"


class ELEC(StrEnum):
    LOGIN = "https://app.bupt.edu.cn/buptdf/wap/default/chong"
    CHECK = "https://app.bupt.edu.cn/buptdf/wap/default/chong"
    SEARCH = "https://app.bupt.edu.cn/buptdf/wap/default/search"



from bs4 import BeautifulSoup


from buptmw.plates.template import Module
from buptmw.constants import CAS as CASE
from buptmw.utlis.auto_retry import auto_retry_network_connections


class CAS(Module):
    def __init__(self, username, password):
        super().__init__()
        self._login(username, password)

    @auto_retry_network_connections
    def _login(self, username, password):
        resp = self.get(url=CASE.LOGIN)
        parsed = BeautifulSoup(resp.text, "lxml")
        varid = parsed.find(attrs={"name": "execution"})["value"]
        post_data = {
            "username": username,
            "password": password,
            "type": "username_password",
            "submit": "LOGIN",
            "_eventId": "submit",
            "execution": varid
        }

        # captcha
        captcha = parsed.find(id="captcha")
        if captcha:
            captcha_img = captcha.find("img").get("src")
            print(captcha_img)

        resp = self.post(url=CASE.LOGIN, data=post_data)
        resp.raise_for_status()


    def check(self):
        resp = self.get(CASE.LOGIN, allow_redirects=False)
        if resp.status_code == 302:
            return True
        return False



from buptmw.constants import UC as UCE
from buptmw.plates.cas import CAS
from buptmw.plates.template import Module_CAS


class UC(Module_CAS):
    def __init__(self, cas: CAS = None):
        super().__init__(cas)
        self._login()

    def _login(self):
        self.get(UCE.LOGIN)



class RequireCASInstance(Exception):
    def __init__(self):
        self.message = "Require CAS instance."
        super().__init__(self.message)

class RequireCASCredential(Exception):
    def __init__(self):
        self.message = "Require CAS credential."
        super().__init__(self.message)



from typing import TypedDict


from buptmw.plates.cas import CAS
from buptmw.plates.uc import UC
from buptmw.plates.ucloud import UCloud
from buptmw.plates.elec import Electric
from buptmw.exception import RequireCASCredential


class CAS_Credential(TypedDict):
    username: str | int
    password: str


class BUPT_Auth:
    def __init__(self, cas: CAS_Credential = None):
        self.cas_credential: CAS_Credential = None
        self.login_CAS(cas)
    
    def login_CAS(self, cas: CAS_Credential = None):
        if cas is not None:
            self.cas_credential = cas

        if self.cas_credential is None:
            raise RequireCASCredential
        self.cas = CAS(cas["username"], cas["password"])

    def get_UC(self) -> UC:
        if not self.cas.check():
            self.login_CAS()
        return UC(self.cas)

    def get_UCloud(self) -> UCloud:
        if not self.cas.check():
            self.login_CAS()
        return UCloud(self.cas)

    def get_Electric(self) -> Electric:
        if not self.cas.check():
            self.login_CAS()
        return Electric(self.cas)



from abc import ABC, abstractmethod
from typing import TYPE_CHECKING
from requests import Session
from copy import deepcopy


from buptmw.constants import SESSION
if TYPE_CHECKING:
    from buptmw.plates.cas import CAS


class Module(Session, ABC):
    def __init__(self):
        super().__init__()
        self.headers["User-Agent"] = SESSION.USER_AGENT

    @abstractmethod
    def _login(self):
        """ Login this module. """
        pass

class Module_CAS(Module):
    def __init__(self, cas: "CAS" = None):
        from buptmw.plates.cas import CAS
        from buptmw.exception import RequireCASInstance
    
        if not isinstance(cas, CAS):
            raise RequireCASInstance()
        super().__init__()
        
        # Copy headers and cookies
        self.headers = deepcopy(cas.headers)
        self.cookies = deepcopy(cas.cookies)
